@@ -1,3 +1,4 @@
+from devpi_common.url import URL
 import json
 
 
@@ -6,20 +7,22 @@ def main(hub, args=None):
     current = hub.current
     args = hub.args
 
-    path = args.path
+    path_url = URL(args.path)
 
     current = hub.current
 
-    if path.startswith(('http://', 'https://')):
-        url = path
-    elif path[0] != "/" and not current.index:
+    if path_url.scheme in ('http', 'https'):
+        url = path_url
+    elif not path_url.path.startswith("/") and not current.index:
         hub.fatal("cannot use relative path without an active index")
     elif current.index:
-        url = current.index_url.addpath(path)
+        url = current.index_url.addpath(path_url.path)
     elif current.root_url:
-        url = current.root_url.addpath(path)
+        url = current.root_url.addpath(path_url.path)
     else:
         hub.fatal("no server currently selected")
+    if path_url.query:
+        url = url.replace(query=path_url.query)
     r = hub.http_api("get", url, quiet=True, check_version=False)
     if hub.args.verbose:
         hub.line("GET REQUEST sent to %s" % url)
