@@ -32,6 +32,11 @@ devpi-server managed index.  For more information see http://doc.devpi.net
 
 hookimpl = HookimplMarker("devpiclient")
 
+try:
+    PermissionError
+except NameError:
+    PermissionError = OSError
+
 
 def main(argv=None):
     if argv is None:
@@ -177,8 +182,8 @@ class Hub:
         else:
             self._last_http_stati.append(r.status_code)
 
-        if r.url != url:
-            self.info("*redirected: %s" %(r.url,))
+        if r.url.rstrip('/') != url.rstrip('/'):
+            self.info("*redirected from %s to %s" % (url, r.url))
 
         reply = HTTPReply(r)
 
@@ -232,7 +237,17 @@ class Hub:
     def workdir(self, prefix='devpi-'):
         def remove_readonly(func, path, excinfo):
             os.chmod(path, stat.S_IWRITE)
-            func(path)
+            count = 3
+            while count:
+                try:
+                    func(path)
+                    return
+                except PermissionError:
+                    count = count - 1
+                    if count == 0:
+                        raise
+                    # wait a moment for other processes to finish
+                    time.sleep(1)
 
         workdir = py.path.local(
             mkdtemp(prefix=prefix))
@@ -404,10 +419,14 @@ class Hub:
         env = os.environ.copy()
         if extraenv is not None:
             env.update({k: str(v) for k, v in extraenv.items()})
-        ret = subprocess.call(args, env=env, **kwargs)
-        if ret != 0:
-            self.fatal_code("command failed", code=ret)
-        return ret
+        assert kwargs.get('stderr') != subprocess.PIPE
+        assert kwargs.get('stdout') != subprocess.PIPE
+        try:
+            subprocess.check_call(args, env=env, **kwargs)
+        except subprocess.CalledProcessError as e:
+            self.fatal_code("command failed", code=e.returncode)
+            return e.returncode
+        return 0
 
     def line(self, *msgs, **kwargs):
         msg = " ".join(map(str, msgs))
@@ -434,8 +453,8 @@ class Hub:
         title = {
             'devpi': 'Devpi',
             'pypi': 'PyPI'}[password.split('-', 1)[0]]
-        if sys.version_info[:2] < (3, 6):
-            # Python below 3.6 not supported by pypitoken
+        if sys.version_info[:2] < (3, 7):
+            # Python below 3.7 not supported by pypitoken
             # so don't bother checking or mentioning it
             return password
         try:
@@ -443,22 +462,32 @@ class Hub:
         except ImportError:
             self.info(
                 "Possibly detected a %s token as password. "
-                "If you install 'pypitoken', "
+                "If you install 'pypitoken>=6.0.1', "
                 "a unique derived token can be created for enhanced security" %
                 title)
             return password
         try:
             token = pypitoken.Token.load(password)
+            legacy = any(
+                isinstance(x, pypitoken.LegacyNoopRestriction)
+                for x in token.restrictions)
             if now is None:
                 now = int(time.time())
-            token.restrict(
-                projects=[project],
-                not_before=now - 1,
-                not_after=now + 60)
+            if legacy:
+                token.restrict(
+                    legacy_project_names=[project],
+                    legacy_not_before=now - 1,
+                    legacy_not_after=now + 60)
+            else:
+                token.restrict(
+                    project_names=[project],
+                    not_before=now - 1,
+                    not_after=now + 60)
             self.info(
                 "Used 'pypitoken' to create a unique %s token "
                 "valid for 60 seconds for upload to the %r project." % (
                     title, project))
+            self.debug("token info:", now, token.restrictions)
             return token.dump()
         except pypitoken.PyPITokenException as e:
             msg = ''.join(traceback.format_exception_only(e.__class__, e)).strip()
