@@ -237,6 +237,9 @@ def test_derive_token_invalid_token(prefix):
     msgs = []
 
     class MockHub:
+        def info(self, msg):
+            msgs.append(msg)
+
         def warn(self, msg):
             msgs.append(msg)
     hub = MockHub()
@@ -246,10 +249,10 @@ def test_derive_token_invalid_token(prefix):
     assert "can not parse it" in msg
 
 
-@pytest.mark.skipif("sys.version_info < (3, 6)")
+@pytest.mark.skipif("sys.version_info < (3, 7)")
 def test_derive_token():
-    import pypitoken.token
-    token = pypitoken.token.Token.create(
+    import pypitoken
+    token = pypitoken.Token.create(
         domain="example.com",
         identifier="devpi",
         key="secret")
@@ -257,6 +260,9 @@ def test_derive_token():
     msgs = []
 
     class MockHub:
+        def debug(self, *msg):
+            pass
+
         def info(self, msg):
             msgs.append(msg)
     hub = MockHub()
@@ -265,19 +271,52 @@ def test_derive_token():
     assert derived_passwd != passwd
     (msg,) = msgs
     assert "create a unique PyPI token" in msg
-    derived_token = pypitoken.token.Token.load(derived_passwd)
-    assert derived_token.restrictions == [
-        pypitoken.token.ProjectsRestriction(projects=["pkg"]),
-        pypitoken.token.DateRestriction(not_before=9, not_after=70)]
+    derived_token = pypitoken.Token.load(derived_passwd)
+    assert sorted(derived_token.restrictions, key=lambda x: x.__class__.__name__) == [
+        pypitoken.DateRestriction(not_before=9, not_after=70),
+        pypitoken.ProjectNamesRestriction(project_names=["pkg"])]
 
 
-@pytest.mark.skipif("sys.version_info < (3, 6)")
+@pytest.mark.skipif("sys.version_info < (3, 7)")
+def test_derive_legacy_token():
+    import pypitoken
+    token = pypitoken.Token.create(
+        domain="example.com",
+        identifier="devpi",
+        key="secret")
+    token.restrict(legacy_noop=True)
+    passwd = token.dump()
+    msgs = []
+
+    class MockHub:
+        def debug(self, *msg):
+            pass
+
+        def info(self, msg):
+            msgs.append(msg)
+    hub = MockHub()
+    hub.derive_token = Hub.derive_token.__get__(hub)
+    derived_passwd = hub.derive_token(passwd, 'pkg', now=10)
+    assert derived_passwd != passwd
+    (msg,) = msgs
+    assert "create a unique PyPI token" in msg
+    derived_token = pypitoken.Token.load(derived_passwd)
+    assert sorted(derived_token.restrictions, key=lambda x: x.__class__.__name__) == [
+        pypitoken.LegacyDateRestriction(not_before=9, not_after=70),
+        pypitoken.LegacyNoopRestriction(),
+        pypitoken.LegacyProjectNamesRestriction(project_names=["pkg"])]
+
+
+@pytest.mark.skipif("sys.version_info < (3, 7)")
 def test_derive_devpi_token():
     import pypitoken
     passwd = "devpi-AgEAAhFmc2NodWx6ZS1yTlk5a0RuYQAABiBcjsOFkn7_3fn6mFoeJve_cOv-thDRL-4fQzbf_sOGjQ"
     msgs = []
 
     class MockHub:
+        def debug(self, *msg):
+            pass
+
         def info(self, msg):
             msgs.append(msg)
     hub = MockHub()
@@ -286,10 +325,12 @@ def test_derive_devpi_token():
     assert derived_passwd != passwd
     (msg,) = msgs
     assert "create a unique Devpi token" in msg
-    derived_token = pypitoken.token.Token.load(derived_passwd)
-    assert derived_token.restrictions == [
-        pypitoken.token.ProjectsRestriction(projects=["pkg"]),
-        pypitoken.token.DateRestriction(not_before=9, not_after=70)]
+    derived_token = pypitoken.Token.load(derived_passwd)
+    assert derived_token.restrictions == sorted(
+        [
+            pypitoken.DateRestriction(not_before=9, not_after=70),
+            pypitoken.ProjectNamesRestriction(project_names=["pkg"])],
+        key=lambda x: x.__class__.__name__)
 
 
 class TestPush:
